import akasha
import pytest


@pytest.mark.summary
def test_Summary():
    summ = akasha.summary(
        "openai:gpt-3.5-turbo",
        sum_type="map_reduce",
        sum_len=1000,
        language="en",
        keep_logs=True,
        max_input_tokens=3000,
        chunk_size=501,
        chunk_overlap=41,
    )

    assert summ.verbose is False
    assert summ.chunk_size == 501
    assert summ.chunk_overlap == 41
    assert summ.max_input_tokens == 3000

    text = summ(
        content=["https://github.com/iii-org/akasha"],
        sum_type="map_reduce",
        sum_len=300,
    )

    assert isinstance(text, str)

    return



import pytest
import akasha

install_requires = [
    "pypdf",
    "langchain>=0.1.0",
    "chromadb==0.4.14",
    "openai==0.27",
    "tiktoken",
    "scikit-learn<1.3.0",
    "jieba==0.42.1",
    "llama-cpp-python==0.2.6",
    "auto-gptq==0.3.1",
    "tqdm==4.65.0",
    "docx2txt==0.8",
    "rouge==1.0.1",
    "rouge-chinese==1.0.3",
    "click",
    "tokenizers>=0.13.3",
    "streamlit==1.28.2",
    "streamlit_option_menu==0.3.6",
]


def base_search(
    query_embeds, docs_embeds, k: int, relevancy_threshold: float, logs: dict
):
    from scipy.spatial.distance import euclidean

    distance = [
        [euclidean(query_embeds, docs_embeds[idx]), idx]
        for idx in range(len(docs_embeds))
    ]
    distance = sorted(distance, key=lambda x: x[0])

    # print(distance) #if (1 - dist) >= relevancy_threshold
    return [idx for dist, idx in distance[:k]]


# def base_model(prompt: str):
#     import openai
#     from langchain_community.chat_models import ChatOpenAI

#     model = akasha.helper.handle_model("gpt-3.5-turbo",
#                                        False,
#                                        0.0,
#                                        max_output_tokens=1000)
#     ret = akasha.helper.call_model(model, prompt)

#     return ret


def base_embed(texts: list) -> list:
    import numpy as np

    embeds = np.array([[0.1 * 1000] for _ in range(len(texts))])

    return embeds


@pytest.fixture
def base_line():
    ak = akasha.RAG(
        embeddings="openai:text-embedding-3-small",
        model="openai:gpt-3.5-turbo",
        verbose=False,
        chunk_size=500,
        max_input_tokens=3010,
        temperature=0.15,
        system_prompt="You are the expert of Market Intelligence and Consulting Institute, please answer the following questions: ",
    )
    return ak


@pytest.mark.akasha
def test_RAG(base_line: akasha.RAG):
    ak = base_line
    query = "五軸是甚麼?"
    assert ak.verbose is False
    assert ak.search_type == "auto"
    assert ak.chunk_size == 500
    assert ak.max_input_tokens == 3010
    assert ak.temperature == 0.15
    assert (
        ak.system_prompt
        == "You are the expert of Market Intelligence and Consulting Institute, please answer the following questions: "
    )
    response = ak("./docs/mic", query)
    assert isinstance(response, str)

    return


@pytest.mark.akasha
def test_ask():
    ak = akasha.ask(
        keep_logs=True,
        verbose=True,
        max_input_tokens=3000,
        stream=True,
    )
    res = ak("此requirement中chromadb的版本為何?", install_requires)
    ret = ""
    for r in res:
        ret += r
        continue
    assert ak.max_input_tokens == 3000
    assert "0.4.14" in ret

    return



import pytest
import akasha
from typing import Tuple


@pytest.fixture
def base_line():
    eva = akasha.eval(
        embeddings="openai:text-embedding-3-small",
        model="openai:gpt-3.5-turbo",
        verbose=True,
        search_type="bm25",
        chunk_size=500,
        max_input_tokens=2468,
        temperature=0.15,
        keep_logs=True,
    )
    doc_path = "./docs/mic/"
    return eva, doc_path


@pytest.mark.eval
def test_Model_Eval(base_line: Tuple[akasha.eval, str]):
    eva, doc_path = base_line

    assert eva.verbose is True
    assert eva.search_type == "bm25"
    assert eva.chunk_size == 500
    assert eva.max_input_tokens == 2468
    assert eva.temperature == 0.15

    ql, al = eva.create_questionset(
        doc_path, question_type="compared", question_num=2, question_style="essay"
    )
    # com_name = eva.logs[eva.timestamp_list[-1]]["questionset_path"]
    assert len(ql) == len(al)

    ql, al = eva.create_questionset(
        doc_path, question_type="summary", question_num=2, question_style="essay"
    )
    # sum_name = eva.logs[eva.timestamp_list[-1]]["questionset_path"]
    assert len(ql) == len(al)

    ql, al = eva.create_questionset(
        doc_path, question_num=2, question_type="fact", question_style="essay"
    )
    f1_name = eva.logs[eva.timestamp_list[-1]]["questionset_path"]
    assert len(ql) == len(al)

    ql, al = eva.create_questionset(
        doc_path, question_num=2, question_style="single_choice"
    )
    f2_name = eva.logs[eva.timestamp_list[-1]]["questionset_path"]
    assert len(ql) == len(al)

    avg_rouge, avg_llm_score, tokens = eva.evaluation(
        f1_name, doc_path, question_style="essay"
    )
    assert isinstance(avg_rouge, float)
    assert isinstance(avg_llm_score, float)
    assert isinstance(tokens, list)

    assert 0 <= avg_rouge <= 1
    assert 0 <= avg_llm_score <= 1

    cor_rate, tokens = eva.evaluation(
        f2_name, doc_path, question_style="single_choice", prompt_format_type="chat_gpt"
    )
    assert isinstance(cor_rate, float)
    assert isinstance(tokens, list)

    assert 0 <= cor_rate <= 1

    return



import pytest
import akasha


def today_f():
    from datetime import datetime

    now = datetime.now()

    return "today's date: " + str(now.strftime("%Y-%m-%d %H:%M:%S"))


today_tool = akasha.create_tool(
    "This is the tool to get today's date, the tool don't have any input parameter.",
    today_f,
    "today_date_tool",
)


@pytest.mark.agent
def test_agent():
    agent = akasha.agents(
        tools=[today_tool], temperature=1.0, verbose=True, keep_logs=True
    )

    res = agent("今天幾月幾號?")

    assert isinstance(agent.tool_name_str, str)
    assert isinstance(res, str)

    return



import pytest
import akasha  # noqa: F401
from akasha.helper import handle_embeddings

EMB_OBJ = handle_embeddings("openai:text-embedding-3-small", False, "")
CHUNK_SIZE = 1000
CERTAIN_FILE = "docs/mic/20230224_製造業機廠鏈智慧應用發展態勢.pdf"


@pytest.mark.db
def test_create_db():
    from akasha.utils.db.create_db import create_directory_db, create_single_file_db

    suc, ign = create_directory_db("docs/mic", EMB_OBJ, CHUNK_SIZE)

    assert suc is True
    assert ign == []

    suc = create_single_file_db(CERTAIN_FILE, EMB_OBJ, CHUNK_SIZE)

    assert suc is True

    return


@pytest.mark.db
def test_load_extract_db():
    from akasha.utils.db import process_db, extract_db_by_file

    db, ign = process_db("docs/mic", EMB_OBJ, CHUNK_SIZE)

    assert len(db.get_ids()) > 0

    assert ign == []

    new_db = extract_db_by_file(db, [CERTAIN_FILE])

    assert len(db.get_ids()) > len(new_db.get_ids())

    return


@pytest.mark.db
def test_delete_file_db():
    from akasha.utils.db import delete_documents_by_file

    delete_num = delete_documents_by_file(CERTAIN_FILE, EMB_OBJ, CHUNK_SIZE)

    assert delete_num > 0

    return
